<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */

use Application\Factory\InvokableServiceFactory;
use Application\View\Helper\ViewHelperFactory;
use Application\Controller\IndexControllerFactory;
use Files\Controller\IndexController;
use Files\Format\Manager;
use Files\Archiver;
use Files\Service\File;
use Application\Config\Services;

return [
    'archives' => [
        'max_input_size'     => 512 * 1024 * 1024, // 512M (must be in bytes)
        'archive_timeout'    => 1800,              // 30 minutes (must be in seconds)
        'cache_lifetime'     => 60 * 60 * 24,      // time to keep archives before deleting them (in seconds)
        'compression_level'  => 1                  // should be between 0 (no compression) and 9 (maximum compression)
    ],
    'files' => [
        'max_size'           => 1024 * 1024,       // 1MB (must be in bytes)
        'download_timeout'   => 1800,              // Time allowed before download of a non archive file will error
                                                   // default 30 minutes (must be in seconds)
    ],
    'diffs' => [
        'max_diffs'          => 1500        // maximum number of diff lines without manual user override
    ],
    'xhprof' => [
        'ignored_routes' => ['archive', 'download', 'view']
    ],
    'router' => [
        'routes' => [
            'file' => [
                'type' => 'Application\Router\Regex',
                'options' => [
                    'regex'    => '/files?(/(?P<path>.*))?',
                    'spec'     => '/files/%path%',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'file',
                        'path'       => null
                    ],
                ],
            ],
            'view' => [
                'type' => 'Application\Router\Regex',
                'options' => [
                    'regex'    => '/view(/(?P<path>.*))?',
                    'spec'     => '/view/%path%',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'file',
                        'path'       => null,
                        'view'       => true
                    ],
                ],
            ],
            'download' => [
                'type' => 'Application\Router\Regex',
                'options' => [
                    'regex'    => '/downloads?(/(?P<path>.*))?',
                    'spec'     => '/downloads/%path%',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'file',
                        'path'       => null,
                        'download'   => true
                    ],
                ],
            ],
            'archive' => [
                'type' => 'Application\Router\Regex',
                'options' => [
                    'regex'    => '/archives?/(?P<path>.+)\.zip',
                    'spec'     => '/archives/%path%.zip',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'archive'
                    ],
                ],
            ],
            'archive-status' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/archive-status/:digest[/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'archive'
                    ],
                ],
            ],
            'diff' => [
                'type' => 'Zend\Router\Http\Literal',
                'options' => [
                    'route'    => '/diff',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'diff',
                    ],
                ],
            ],
        ],
    ],
    'controllers' => [
        'factories' => [
            IndexController::class => IndexControllerFactory::class
        ],
    ],
    'view_manager' => [
        'template_path_stack'   => [
            __DIR__ . '/../view',
        ],
    ],
    'view_helpers' => [
        'factories' => array_fill_keys(
            [
                ViewHelperFactory::FILE_SIZE,
                ViewHelperFactory::DECODE_FILE_SPEC,
                ViewHelperFactory::DECODE_SPEC,
                ViewHelperFactory::FILE_TYPE_VIEW,
            ],
            ViewHelperFactory::class
        )
    ],
    'service_manager' => [
        'aliases'   => [
            Manager::FORMATS       => Manager::class,
            Archiver::ARCHIVER     => Archiver::class,
            Services::FILE_SERVICE => File::class
        ],
        'factories' => [
            Manager::class  => InvokableServiceFactory::class,
            Archiver::class => InvokableServiceFactory::class,
            File::class     => InvokableServiceFactory::class
        ],
    ],
];
