<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */

namespace Redis\Model;

/**
 * Interface for all Swarm DAOs to implement
 * @package Application\Model
 */

interface IModelDAO extends \Application\Model\IModelDAO
{

    /**
     * Search the redis for any records.
     *
     * @param string            $id         the id of the record, for example a user id. Cache key prefixes will be
     *                                      used as per the implementing class
     * @param string            $hashKey    the hashKey
     */
    public function search($id, $hashKey);

    /**
     * Populate the cache with the objects returned from the Perforce Server. Ensuring that any cache items for the
     * model is removed before we populate to ensure we do not leave random id within the cache.
     */
    public function populate();

    /**
     * Remove all keys for the namespace and prefix configured for this DAO.
     */
    public function invalidate();
}
