<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */
namespace TestIntegration\Filter;

use Application\Config\ConfigManager;
use Application\I18n\TranslatorFactory;
use Interop\Container\ContainerInterface;
use Zend\Validator\AbstractValidator;
use TestIntegration\Model\TestDefinition as Model;
use Application\Config\ConfigException;

/**
 * Class NamesValidator to validate uniqueness of global test definition names
 * @package TestIntegration\Filter
 */
class NamesValidator extends AbstractValidator
{
    const DUPLICATE_NAMES = 'duplicateNames';
    private $services;
    private $config;

    /**
     * NamesValidator constructor.
     * @param ContainerInterface $services  application services
     */
    public function __construct(ContainerInterface $services)
    {
        parent::__construct();
        $this->services = $services;
        $this->config   = $services->get(ConfigManager::CONFIG);
    }

    /**
     * Validates 'global_tests' within the application configuration to ensure unique names
     * @param null $value
     * @return bool true if valid
     * @throws ConfigException
     */
    public function isValid($value = null)
    {
        $configuredTests = ConfigManager::getValue($this->config, ConfigManager::GLOBAL_TESTS, []);
        $names           = [];
        foreach ($configuredTests as $configuredTest) {
            $names[] = $configuredTest[Model::FIELD_NAME];
        }
        $duplicates = array_unique(array_diff_assoc($names, array_unique($names)));
        // This particular name is only invalid if there are duplicates and this name is in the duplicates found
        if ($duplicates && in_array($value, $duplicates)) {
            $translator = $this->services->get(TranslatorFactory::SERVICE);

            $this->abstractOptions['messages'][self::DUPLICATE_NAMES] =
                $translator->t(
                    "Test configuration names must be unique, duplicates found [%s]",
                    [implode(', ', $duplicates)]
                );
            return false;
        }
        return true;
    }
}
