<?php


/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */

namespace TestIntegration\Service;

use Application\Factory\InvokableService;
use P4\Model\Fielded\Iterator;
use Projects\Model\Project;
use Record\Key\AbstractKey;
use TestIntegration\Model\ITestDefinition;
use TestIntegration\Model\ITestRun as ITestRunModel;

/**
 * Interface ITestExecutor
 * @package TestIntegration\Service
 */
interface ITestExecutor extends InvokableService
{
    const NAME = 'testExecutorService';
    // Test run name for project tests is 'project:<project id>:<test name>
    const PROJECT_TEST_SEPARATOR = ':';
    const PROJECT_PREFIX         = 'project' . self::PROJECT_TEST_SEPARATOR;
    const PROJECT_TEST_FORMAT    = self::PROJECT_PREFIX . "%s" . self::PROJECT_TEST_SEPARATOR . "%s";

    /**
     * Runs all tests associated with a given review and its projects
     * @param AbstractKey $review           review, against which, global test will be run
     * @param array       $currentProjects  maps each affected project's id to its branch ids
     *                                      Ex. ['proj1' => ['branchA', 'branchB'], 'proj2' => ['branchY', 'branchX']]
     * @param Iterator    $projects         instances of Project models, directly associated with the review
     */
    public function runReviewTests(AbstractKey $review, array $currentProjects, Iterator $projects);

    /**
     * Runs all the global tests
     */
    public function runGlobalTests();

    /**
     * Runs the project tests associated with a review
     */
    public function runProjectTests();

    /**
     * Runs a single global test
     * @param ITestDefinition $testDefinition   global test definition, stored in the config
     * @param array           $fields           key-value pairs, used to build the query string for the CI request
     * @param ITestRunModel   $testRun          test run record
     */
    public function runGlobalTest(ITestDefinition $testDefinition, array $fields, ITestRunModel $testRun);

    /**
     * Runs a single project test
     * @param Project         $project          project object with test definition information
     * @param array           $fields           key-value pairs, used to build the query string for the CI request
     * @param ITestRunModel   $testRun          test run record
     * @param array           $testStartTimes   holds the start times of all the project tests
     */
    public function runProjectTest(Project $project, array $fields, ITestRunModel $testRun, array &$testStartTimes);
}
